@@ -1,21 +1,33 @@
 import os
 import re
 import sys
-from typing import List, Type, Union
+from typing import List, Optional, Type, Union
 
 import elasticsearch
 import elasticsearch_dsl
 import fatcat_openapi_client
 import requests
+from dynaconf import Dynaconf
 from fatcat_openapi_client import ContainerEntity, DefaultApi, ReleaseEntity
+from fatcat_openapi_client.rest import ApiException
 
 from fuzzycat.entities import entity_from_dict, entity_from_json
 
+settings = Dynaconf(envvar_prefix="FUZZYCAT")
+FATCAT_API_URL = settings.get("FATCAT_API_URL", "https://api.fatcat.wiki/v0")
 
-def match_release_fuzzy(release: ReleaseEntity, size=5, es=None) -> List[ReleaseEntity]:
+
+def match_release_fuzzy(
+    release: ReleaseEntity,
+    size: int = 5,
+    es: Optional[Union[str, Type[elasticsearch.client.Elasticsearch]]] = None,
+    api: DefaultApi = None,
+) -> List[ReleaseEntity]:
     """
     Given a release entity, return a number similar release entities from
     fatcat using Elasticsearch.
+
+    TODO: rename "es" parameter to "es_client", which would be clearer
     """
     assert isinstance(release, ReleaseEntity)
 
@@ -26,33 +38,34 @@ def match_release_fuzzy(release: ReleaseEntity, size=5, es=None) -> List[Release
         es = elasticsearch.Elasticsearch([es])
     if es is None:
         es = elasticsearch.Elasticsearch()
+    if api is None:
+        api = public_api(FATCAT_API_URL)
 
     # Try to match by external identifier.
+    # TODO: use api, ability to disable; benchmark
     ext_ids = release.ext_ids
-    attrs = {
-        "doi": "doi",
-        "wikidata_qid": "wikidata_qid",
-        "isbn13": "isbn13",
-        "pmid": "pmid",
-        "pmcid": "pmcid",
-        "core": "code_id",
-        "arxiv": "arxiv_id",
-        "jstor": "jstor_id",
-        "ark": "ark_id",
-        "mag": "mag_id",
-    }
-    for attr, es_field in attrs.items():
+    attrs = (
+        "doi",
+        "wikidata_qid",
+        "isbn13",
+        "pmid",
+        "pmcid",
+        "core",
+        "arxiv",
+        "jstor",
+        "ark",
+        "mag",
+        "doaj",
+        "dblp",
+        "oai",
+    )
+    for attr in attrs:
         value = getattr(ext_ids, attr)
         if not value:
             continue
-        s = (elasticsearch_dsl.Search(using=es,
-                                      index="fatcat_release").query("term", **{
-                                          es_field: value
-                                      }).extra(size=size))
-        print(s)
-        resp = s.execute()
-        if len(resp) > 0:
-            return response_to_entity_list(resp, entity_type=ReleaseEntity)
+        r = api.lookup_release(**{attr: value})
+        if r:
+            return r
 
     body = {
         "query": {
@@ -67,7 +80,7 @@ def match_release_fuzzy(release: ReleaseEntity, size=5, es=None) -> List[Release
     }
     resp = es.search(body=body, index="fatcat_release")
     if resp["hits"]["total"] > 0:
-        return response_to_entity_list(resp, entity_type=ReleaseEntity)
+        return response_to_entity_list(resp, entity_type=ReleaseEntity, api=api)
 
     # Get fuzzy.
     # https://www.elastic.co/guide/en/elasticsearch/reference/current/common-options.html#fuzziness
@@ -85,7 +98,7 @@ def match_release_fuzzy(release: ReleaseEntity, size=5, es=None) -> List[Release
     }
     resp = es.search(body=body, index="fatcat_release")
     if resp["hits"]["total"] > 0:
-        return response_to_entity_list(resp, entity_type=ReleaseEntity)
+        return response_to_entity_list(resp, entity_type=ReleaseEntity, api=api)
 
     # TODO: perform more queries on other fields.
     return []
@@ -112,7 +125,7 @@ def retrieve_entity_list(
     that are accessible.
     """
     if api is None:
-        api = public_api("https://api.fatcat.wiki/v0")
+        api = public_api(FATCAT_API_URL)
     result = []
     if entity_type == ReleaseEntity:
         for id in ids:
@@ -121,9 +134,7 @@ def retrieve_entity_list(
                 result.append(re)
             except ApiException as exc:
                 if exc.status == 404:
-                    print("[err] failed to retrieve release entity: {} (maybe stale index)".format(
-                        id),
-                          file=sys.stderr)
+                    print("[err] failed to retrieve release entity: {}".format(id), file=sys.stderr)
                 else:
                     print("[err] api failed with {}: {}".format(exc.status, exc.message),
                           file=sys.stderr)
@@ -134,10 +145,8 @@ def retrieve_entity_list(
                 result.append(re)
             except ApiException as exc:
                 if exc.status == 404:
-                    print(
-                        "[err] failed to retrieve container entity: {} (maybe stale index)".format(
-                            id),
-                        file=sys.stderr)
+                    print("[err] failed to retrieve container entity: {}".format(id),
+                          file=sys.stderr)
                 else:
                     print("[err] api failed with {}: {}".format(exc.status, exc.message),
                           file=sys.stderr)
@@ -147,7 +156,7 @@ def retrieve_entity_list(
     return result
 
 
-def response_to_entity_list(response, size=5, entity_type=ReleaseEntity):
+def response_to_entity_list(response, size=5, entity_type=ReleaseEntity, api: DefaultApi = None):
     """
     Convert an elasticsearch result to a list of entities. Accepts both a
     dictionary and an elasticsearch_dsl.response.Response.
@@ -156,10 +165,10 @@ def response_to_entity_list(response, size=5, entity_type=ReleaseEntity):
     """
     if isinstance(response, dict):
         ids = [hit["_source"]["ident"] for hit in response["hits"]["hits"]][:size]
-        return retrieve_entity_list(ids, entity_type=entity_type)
+        return retrieve_entity_list(ids, entity_type=entity_type, api=api)
     elif isinstance(response, elasticsearch_dsl.response.Response):
         ids = [hit.to_dict().get("ident") for hit in response]
-        return retrieve_entity_list(ids, entity_type=entity_type)
+        return retrieve_entity_list(ids, entity_type=entity_type, api=api)
     else:
         raise ValueError("cannot convert {}".format(response))
 
@@ -179,7 +188,7 @@ def anything_to_entity(
         ReleaseEntity: "release",
     }
     if not entity_type in names:
-        raise ValueError("cannot convert {} - only: {}".format(entity_type, names.keys()))
+        raise ValueError("cannot convert {}, only: {}".format(entity_type, names.keys()))
     entity_name = names[entity_type]
 
     if s is None:
@@ -199,6 +208,7 @@ def anything_to_entity(
             raise ValueError("entity not found: {}".format(url))
 
     if re.match("[0-9]{4}(-)?[0-9]{3,3}[0-9xx]", s):
+        # TODO: make index name configurable
         url = "{}/fatcat_{}/_search?q=issns:{}".format(es_url, entity_name, s)
         doc = requests.get(url).json()
         if doc["hits"]["total"] == 1:
